/* Demo DIF-to-C code generation (software synthesis).

Copyright (c) 1999-2008
Maryland DSPCAD Research Group, The University of Maryland at College Park 

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the above
copyright notice and the following two paragraphs appear in all copies
of this software.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

*/

import mapss.dif.softsyn.c.DIFtoC;
import mapss.dif.softsyn.scheduler.Scheduler;
import mapss.dif.softsyn.scheduler.SDFScheduler;

import mapss.dif.DIFHierarchy;
import mapss.dif.language.Reader;
import mapss.dif.language.DIFLanguageException;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;

//////////////////////////////////////////////////////////////////////////
//// DemoDIFtoC
/** Demo DIF-to-C code generation (software synthesis).<p>
<code>
DemoDIFtoC <i>DIFtoCClass difFile CFile SchedulerClass SchedStrategyClass
[-f] [-dppo dppoClass] [-mas masClass]</i>
</code><p>
-f for flatten scheduling

@author Chia-Jui Hsu
@version $Id: DemoDIFtoC.java 1688 2007-05-09 02:22:30Z jerryhsu $
*/

public class DemoDIFtoC {
    public static void main(String[] args) {
        try {

            if (args.length < 5) {
                throw new RuntimeException("DemoDIFtoC DIFtoCClass "
                        + "difFile CFile SchedulerClass SchedStrategyClass "
                        + "[-f] [-dppo dppoClass] [-mas masClass]");
            }
            
            Class DIFtoCClass = Class.forName(args[0]);
            
            Reader reader = new Reader(args[1]);
            reader.compile();
            DIFHierarchy topHierarchy = reader.getTopHierarchy();

            Class schedulerClass = Class.forName(args[3]);
            Scheduler scheduler = (Scheduler)schedulerClass.getConstructor(null)
                    .newInstance(null);
            if (scheduler instanceof SDFScheduler) {
                ((SDFScheduler)scheduler).setStrategy(args[4]);
            }
            
            Class[] paramTypes = {
                    Class.forName("mapss.dif.DIFHierarchy"), 
                    Class.forName("mapss.dif.softsyn.scheduler.Scheduler"),
                    Class.forName("java.lang.String")};
            Object[] params = {topHierarchy, scheduler, args[2]};

            DIFtoC synthesizer = (DIFtoC)DIFtoCClass.getConstructor(paramTypes).newInstance(params);
            //synthesizer.setIteration(600);
            
            int idx = 5;
            while (idx < args.length) {
                String arg = args[idx];
                idx++;
                if (arg.equals("-f")) {
                    synthesizer.setFlatten(true);
                } else if (arg.equals("-dppo")) {
                    String dppoClass = args[idx];
                    idx++;
                    if (scheduler instanceof SDFScheduler) {
                        ((SDFScheduler)scheduler).setDPPO(dppoClass);
                    }
                } else if (arg.equals("-mas")) {
                    String masClass = args[idx];
                    idx++;
                    if (scheduler instanceof SDFScheduler) {
                        ((SDFScheduler)scheduler).setMAS(masClass);
                    }
                } else {
                    throw new RuntimeException("Illegal option.");
                }
            }
                        
            synthesizer.generate();

        } catch (DIFLanguageException exp) {
            throw new RuntimeException(exp.getMessage());
        } catch (IOException exp) {
            throw new RuntimeException(exp.getMessage());
        } catch (ClassNotFoundException exp) {
            throw new RuntimeException(exp.getMessage());
        } catch (NoSuchMethodException exp) {
            throw new RuntimeException(exp.getMessage());
        } catch (InstantiationException exp) {
            throw new RuntimeException(exp.getMessage());
        } catch (IllegalAccessException exp) {
            throw new RuntimeException(exp.getMessage());
        } catch (InvocationTargetException exp) {
            throw new RuntimeException(exp.getMessage());
        }
    }
}


