/* BMP file reader

Copyright (c) 1999-2008
Maryland DSPCAD Research Group, The University of Maryland at College Park 

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the above
copyright notice and the following two paragraphs appear in all copies
of this software.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

*/

//////////////////////////////////////////////////////////////////////////
//// BMPRead.c
/** BMP file reader
This function output r g b component in three array (matrix) in row-by-row scan.
row and column specifies the image dimension.

rate
r: row*column (short)
g: row*column (short)
b: row*column (short)
  
@author Chia-Jui Hsu
@version $Id: BMPRead.c 1688 2007-05-09 02:22:30Z jerryhsu $
*/

#include <stdio.h>

void BMPRead(short *r, short *g, short *b, FILE *file, int row, int column) {
    char type[2];                            /* Magic identifier            */
    unsigned int size;                       /* File size in bytes          */
    unsigned short reserved1, reserved2;
    unsigned int offset;                     /* Offset to image data, bytes */
    unsigned int hsize;               /* Header size in bytes      */
    int width,height;                /* Width and height of image */
    unsigned short planes;       /* Number of colour planes   */
    unsigned short bits;         /* Bits per pixel            */
    unsigned int compression;        /* Compression type          */
    unsigned int imagesize;          /* Image size in bytes       */
    int xresolution,yresolution;     /* Pixels per meter          */
    unsigned int ncolours;           /* Number of colours         */
    unsigned int importantcolours;   /* Important colours         */

    int i,j;
    unsigned char blue;
    unsigned char green;
    unsigned char red;
    int position;
    
    fread(type, sizeof(char), 2, file);
    fread(&size, sizeof(unsigned int), 1, file);
    fread(&reserved1, sizeof(unsigned short), 1, file);
    fread(&reserved2, sizeof(unsigned short), 1, file);
    fread(&offset, sizeof(unsigned int), 1, file);
    fread(&hsize, sizeof(unsigned int), 1, file);
    fread(&width, sizeof(int), 1, file);
    fread(&height, sizeof(int), 1, file);
    fread(&planes, sizeof(unsigned short), 1, file);
    fread(&bits, sizeof(unsigned short), 1, file);
    fread(&compression, sizeof(int), 1, file);
    fread(&imagesize, sizeof(int), 1, file);
    fread(&xresolution, sizeof(int), 1, file);
    fread(&yresolution, sizeof(int), 1, file);
    fread(&ncolours, sizeof(int), 1, file);
    fread(&importantcolours, sizeof(int), 1, file);

    /*printf("type: ");
    printf(type);
    printf("\n");
    printf("size: %d\n", size);
    printf("reserved1: %d\n", reserved1);
    printf("reserved2: %d\n", reserved2);
    printf("offset: %d\n", offset);
    printf("hsize: %d\n", hsize);
    printf("width: %d\n", width);
    printf("height: %d\n", height);
    printf("planes: %d\n", planes);
    printf("bits: %d\n", bits);
    printf("compression: %d\n", compression);
    printf("imagesize: %d\n", imagesize);
    printf("xresolution: %d\n", xresolution);
    printf("yresolution: %d\n", yresolution);
    printf("ncolours: %d\n", ncolours);
    printf("importantcolours: %d\n", importantcolours);*/
    
    if (row != height || column != width) {
        printf("img size error");
        exit(1);
    }
    
    for (i=0; i<height; i++) {
        for (j=0; j<width; j++) {
            position = (width - 1 - i) * width + j;
            fread(&blue, sizeof(unsigned char), 1, file);
            fread(&green, sizeof(unsigned char), 1, file);
            fread(&red, sizeof(unsigned char), 1, file);
            r[position] = (short)red;
            g[position] = (short)green;
            b[position] = (short)blue;
        }
    }
}


