/* Blocking for short matrix

Copyright (c) 1999-2008
Maryland DSPCAD Research Group, The University of Maryland at College Park 

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the above
copyright notice and the following two paragraphs appear in all copies
of this software.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

*/

//////////////////////////////////////////////////////////////////////////
//// Blocking.c
/** Blocking for short matrix
The row*col input matrix is blocked such that the output consists of row-by-row 
scanned block with size brow*bcol and in each block is also row-by-row scanned.

rate
input: row*col (short)
output: row*col (short)
  
@author Chia-Jui Hsu
@version $Id: Blocking.c 1688 2007-05-09 02:22:30Z jerryhsu $
*/

void Blocking(short *input, short *output, int brow, int bcol, int row, int col) {
    int nr, nc, bsize, i,j,x,y;
    if (row % brow != 0 || col % bcol != 0) {
        printf("brow or bcol cannot divide row or col");
        exit(1);
    }
    nr = (int)(row/brow);
    nc = (int)(col/bcol);
    bsize = brow*bcol;

    for (i=0; i<nr; i++) {
        for (j=0; j<nc; j++) {
            
            for (x=0; x<brow; x++) {
                for (y=0; y<bcol; y++) {
                    output[(i*nc+j)*bsize + x*bcol+y] = input[(i*brow+x)*col + j*bcol+y];
                }
            }
            
        }
    }
}

