/* FIR

Copyright (c) 1999-2008
Maryland DSPCAD Research Group, The University of Maryland at College Park 

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the above
copyright notice and the following two paragraphs appear in all copies
of this software.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

*/

//////////////////////////////////////////////////////////////////////////
//// FIR
/** 
1. phaseLength = ceil(tapslength / interpolation) 
2. length = phaseLength + decimation;
3. float feedback[length];
4. mostRecent is initialized to phaseLength
5. feedback is the float array for feedback, and initialize to zero.
6. mostRecent is the feedback index for the feedback array
7. output sample will be multiplied by decimation.

@author Chia-Jui Hsu
@version $Id: FIRAmp.c 1688 2007-05-09 02:22:30Z jerryhsu $
*/

#include <math.h>

void FIRAmp(float* input, float* output, float* taps, float* feedback, int* mostRecent, 
        int interpolation, int decimation, int decPhase, int tapslength) {
    
    int phaseLength = (int)(tapslength / interpolation);
    int length = phaseLength + decimation;
    int phase = decimation - decPhase - 1;
    int i,j;
    int outIdx = 0;

    if ((tapslength % interpolation) != 0) {
        phaseLength++;
    }
    
    for (j=0; j<decimation; j++) {
        if (--(*mostRecent) < 0) {
            (*mostRecent) = length -1;
        }
        feedback[(*mostRecent)] = input[j];
    }
    
    for (j=1; j<=decimation; j++) {
        while (phase < interpolation) {
            float out = 0;
            for (i=0; i<phaseLength; i++) {
                int tapsIdx = i * interpolation + phase;
                int dataIdx = ((*mostRecent) + decimation - j + i) % (length);
                if (tapsIdx < tapslength) {
                    out += taps[tapsIdx] * feedback[dataIdx];
                }
            }
            output[outIdx] = out * decimation;
            outIdx++;
            phase += decimation;
        }
        phase -=interpolation;
    }
}

/* coefficients 1, tapslength = 33
1.032540E-004, 6.546872E-004, 1.806661E-003, 3.668032E-003, 5.555587E-003, 5.499690E-003, 1.894837E-003, -6.561282E-003, -1.833260E-002, -2.734803E-002, -2.632614E-002, -7.170922E-003, 3.334184E-002, 8.907808E-002, 1.482204E-001, 1.933716E-001, 2.098481E-001, 1.933716E-001, 1.482204E-001, 8.907808E-002, 3.334184E-002, -7.170922E-003, -2.632614E-002, -2.734803E-002, -1.833260E-002, -6.561282E-003, 1.894837E-003, 5.499690E-003, 5.555587E-003, 3.668032E-003, 1.806661E-003, 6.546872E-004, 1.032540E-004
*/

/* coefficients 2, tapslength = 33
3.177298e-004, 5.464669e-004, 7.144341e-004, 2.340341e-004, -1.005069e-003, -4.713609e-003, -9.153581e-003, -1.393887e-002, -1.568459e-002, -1.261507e-002, 1.466877e-003, 2.501615e-002, 5.855630e-002, 9.569618e-002, 1.324690e-001, 1.566966e-001, 1.662780e-001, 1.566966e-001, 1.324690e-001, 9.569618e-002, 5.855630e-002, 2.501615e-002, 1.466877e-003, -1.261507e-002, -1.568459e-002, -1.393887e-002, -9.153581e-003, -4.713609e-003, -1.005069e-003, 2.340341e-004, 7.144341e-004, 5.464669e-004, 3.177298e-004
*/

/* coefficients 3, tapslength = 173
1.252648e-004, 5.114661e-004, 1.001673e-003, 1.063325e-003, 3.861077e-004, -4.806018e-004, -6.002877e-004, 6.967818e-005, 4.795735e-004, -4.524777e-006, -5.706942e-004, -2.356764e-004, 4.618192e-004, 2.817336e-004, -5.370126e-004, -5.406652e-004, 3.850344e-004, 6.178104e-004, -3.808617e-004, -8.878338e-004, 1.344012e-004, 9.590940e-004, -1.554649e-005, -1.186917e-003, -3.445968e-004, 1.188361e-003, 5.797071e-004, -1.304681e-003, -1.037629e-003, 1.164207e-003, 1.353305e-003, -1.099216e-003, -1.853418e-003, 7.535834e-004, 2.173830e-003, -4.564298e-004, -2.622436e-003, -1.290178e-004, 2.837224e-003, 6.707296e-004, -3.112169e-003, -1.482410e-003, 3.093138e-003, 2.228045e-003, -3.063967e-003, -3.196395e-003, 2.682102e-003, 4.043510e-003, -2.229406e-003, -5.033956e-003, 1.380657e-003, 5.815421e-003, -4.244204e-004, -6.632571e-003, -9.418402e-004, 7.129426e-003, 2.419942e-003, -7.534268e-003, -4.288947e-003, 7.485648e-003, 6.233107e-003, -7.201318e-003, -8.506126e-003, 6.319549e-003, 1.078006e-002, -5.043537e-003, -1.329017e-002, 2.993166e-003, 1.569206e-002, -3.404384e-004, -1.820890e-002, -3.352858e-003, 2.049652e-002, 8.045191e-003, -2.276652e-002, -1.441162e-002, 2.468327e-002, 2.293987e-002, -2.646304e-002, -3.555827e-002, 2.778189e-002, 5.635159e-002, -2.887207e-002, -1.017023e-001, 2.942982e-002, 3.167858e-001, 4.702912e-001, 3.167858e-001, 2.942982e-002, -1.017023e-001, -2.887207e-002, 5.635159e-002, 2.778189e-002, -3.555827e-002, -2.646304e-002, 2.293987e-002, 2.468327e-002, -1.441162e-002, -2.276652e-002, 8.045191e-003, 2.049652e-002, -3.352858e-003, -1.820890e-002, -3.404384e-004, 1.569206e-002, 2.993166e-003, -1.329017e-002, -5.043537e-003, 1.078006e-002, 6.319549e-003, -8.506126e-003, -7.201318e-003, 6.233107e-003, 7.485648e-003, -4.288947e-003, -7.534268e-003, 2.419942e-003, 7.129426e-003, -9.418402e-004, -6.632571e-003, -4.244204e-004, 5.815421e-003, 1.380657e-003, -5.033956e-003, -2.229406e-003, 4.043510e-003, 2.682102e-003, -3.196395e-003, -3.063967e-003, 2.228045e-003, 3.093138e-003, -1.482410e-003, -3.112169e-003, 6.707296e-004, 2.837224e-003, -1.290178e-004, -2.622436e-003, -4.564298e-004, 2.173830e-003, 7.535834e-004, -1.853418e-003, -1.099216e-003, 1.353305e-003, 1.164207e-003, -1.037629e-003, -1.304681e-003, 5.797071e-004, 1.188361e-003, -3.445968e-004, -1.186917e-003, -1.554649e-005, 9.590940e-004, 1.344012e-004, -8.878338e-004, -3.808617e-004, 6.178104e-004, 3.850344e-004, -5.406652e-004, -5.370126e-004, 2.817336e-004, 4.618192e-004, -2.356764e-004, -5.706942e-004, -4.524777e-006, 4.795735e-004, 6.967818e-005, -6.002877e-004, -4.806018e-004, 3.861077e-004, 1.063325e-003, 1.001673e-003, 5.114661e-004, 1.252648e-004
*/

/* tapslength = 147
1.561517e-05,1.573803e-05,-3.916175e-05,-2.013121e-04,-4.769289e-04,-7.811547e-04,-9.460212e-04,-8.094338e-04,-3.471021e-04,2.502620e-04,6.458242e-04,5.685497e-04,3.756736e-05,-5.864350e-04,-8.148769e-04,-3.991166e-04,4.207815e-04,1.024629e-03,8.635496e-04,-6.042358e-05,-1.093644e-03,-1.371466e-03,-5.238482e-04,9.162409e-04,1.812515e-03,1.302313e-03,-4.093441e-04,-2.045553e-03,-2.184259e-03,-4.668334e-04,1.918083e-03,3.017151e-03,1.685023e-03,-1.296894e-03,-3.601779e-03,-3.139826e-03,9.504808e-05,3.711272e-03,4.640429e-03,1.697292e-03,-3.122718e-03,-5.918007e-03,-3.989470e-03,1.653062e-03,6.645202e-03,6.577715e-03,8.071897e-04,-6.462174e-03,-9.141440e-03,-4.263929e-03,5.007616e-03,1.125122e-02,8.602989e-03,-1.940065e-03,-1.237727e-02,-1.358930e-02,-3.068315e-03,1.187736e-02,1.888204e-02,1.040278e-02,-8.912697e-03,-2.406591e-02,-2.075072e-02,2.126647e-03,2.870115e-02,3.595180e-02,1.165004e-02,-3.236384e-02,-6.284963e-02,-4.462878e-02,3.471336e-02,1.517306e-01,2.560992e-01,2.978102e-01,2.560992e-01,1.517306e-01,3.471336e-02,-4.462878e-02,-6.284963e-02,-3.236384e-02,1.165004e-02,3.595180e-02,2.870115e-02,2.126647e-03,-2.075072e-02,-2.406591e-02,-8.912697e-03,1.040278e-02,1.888204e-02,1.187736e-02,-3.068315e-03,-1.358930e-02,-1.237727e-02,-1.940065e-03,8.602989e-03,1.125122e-02,5.007616e-03,-4.263929e-03,-9.141440e-03,-6.462174e-03,8.071897e-04,6.577715e-03,6.645202e-03,1.653062e-03,-3.989470e-03,-5.918007e-03,-3.122718e-03,1.697292e-03,4.640429e-03,3.711272e-03,9.504808e-05,-3.139826e-03,-3.601779e-03,-1.296894e-03,1.685023e-03,3.017151e-03,1.918083e-03,-4.668334e-04,-2.184259e-03,-2.045553e-03,-4.093441e-04,1.302313e-03,1.812515e-03,9.162409e-04,-5.238482e-04,-1.371466e-03,-1.093644e-03,-6.042358e-05,8.635496e-04,1.024629e-03,4.207815e-04,-3.991166e-04,-8.148769e-04,-5.864350e-04,3.756736e-05,5.685497e-04,6.458242e-04,2.502620e-04,-3.471021e-04,-8.094338e-04,-9.460212e-04,-7.811547e-04,-4.769289e-04,-2.013121e-04,-3.916175e-05,1.573803e-05,1.561517e-05
*/

/* tapslength = 75
-4.681601e-05,-1.086436e-04,-2.264329e-04,-3.957595e-04,-6.307421e-04,-9.101920e-04,-1.230485e-03,-1.442644e-03,-1.570936e-03,-1.465082e-03,-1.081661e-03,-3.294450e-04,7.662807e-04,2.219310e-03,3.702032e-03,5.161901e-03,6.191007e-03,6.538686e-03,5.841503e-03,3.968763e-03,7.293618e-04,-3.424770e-03,-8.336396e-03,-1.319454e-02,-1.726442e-02,-1.949643e-02,-1.908432e-02,-1.503234e-02,-7.240214e-03,4.668056e-03,2.009155e-02,3.827131e-02,5.776025e-02,7.711618e-02,9.432152e-02,1.081667e-01,1.169802e-01,1.200689e-01,1.169802e-01,1.081667e-01,9.432152e-02,7.711618e-02,5.776025e-02,3.827131e-02,2.009155e-02,4.668056e-03,-7.240214e-03,-1.503234e-02,-1.908432e-02,-1.949643e-02,-1.726442e-02,-1.319454e-02,-8.336396e-03,-3.424770e-03,7.293618e-04,3.968763e-03,5.841503e-03,6.538686e-03,6.191007e-03,5.161901e-03,3.702032e-03,2.219310e-03,7.662807e-04,-3.294450e-04,-1.081661e-03,-1.465082e-03,-1.570936e-03,-1.442644e-03,-1.230485e-03,-9.101920e-04,-6.307421e-04,-3.957595e-04,-2.264329e-04,-1.086436e-04,-4.681601e-05
*/

