/* RGB to YCbCr out-of-place

Copyright (c) 1999-2008
Maryland DSPCAD Research Group, The University of Maryland at College Park 

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the above
copyright notice and the following two paragraphs appear in all copies
of this software.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

*/

//////////////////////////////////////////////////////////////////////////
//// RGB2YCbCr.c
/** RGB to YCbCr out-of-place
R,G,B 0~255 to Y,Cb,Cr 0~255 according to Matlab code
Y       0.2568   0.5041   0.0979           R     16
Cb = [  -0.1482 -0.2910   0.4392     ]     G +  128 
Cr      0.4392   -0.3678 -0.0714           B    128
  
rate
r: row*column (short)
g: row*column (short)
b: row*column (short)

y: row*column (short)
cb: row*column (short)
cr: row*column (short)
  
@author Chia-Jui Hsu
@version $Id: RGB2YCbCr.c 1688 2007-05-09 02:22:30Z jerryhsu $
*/

void RGB2YCbCr(short *r, short *g, short *b, short *y, short *cb, short *cr, int row, int column) {
    int length = row * column;
    int i;
    float yC,cbC,crC;
    for (i=0; i<length; i++) {
        yC = 16 + 0.2568 * r[i] + 0.5041 * g[i] + 0.0979 * b[i];
        cbC = 128 + -0.1482 * r[i] + -0.2910 * g[i] + 0.4392 * b[i];
        crC = 128 + 0.4392 * r[i] + -0.3678 * g[i] + -0.0714 * b[i];
        y[i] = (yC-(short)yC) >= 0.5 ? (short)yC + 1: (short)yC;
        cb[i] = (cbC-(short)cbC) >= 0.5 ? (short)cbC + 1: (short)cbC;
        cr[i] = (crC-(short)crC) >= 0.5 ? (short)crC + 1: (short)crC;
    }
}

