/* RGB to YCbCr out-of-place

Copyright (c) 1999-2008
Maryland DSPCAD Research Group, The University of Maryland at College Park 

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the above
copyright notice and the following two paragraphs appear in all copies
of this software.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

*/

//////////////////////////////////////////////////////////////////////////
//// RGB2YCbCr.c
/** RGB to YCbCr out-of-place
R,G,B 0~255 to Y,Cb,Cr 0~255 according to Matlab code
Y       0.2568   0.5041   0.0979           R     16
Cb = [  -0.1482 -0.2910   0.4392     ]     G +  128 
Cr      0.4392   -0.3678 -0.0714           B    128

Because float-point operations in DSP are too expensive, 
so multiply above coefficients by 2^12 = 4096, and shift 12-bit back.

rate
r_y: row*column (short)
g_cb: row*column (short)
b_cr: row*column (short)
  
@author Chia-Jui Hsu
@version $Id: RGB2YCbCrDSPip.c 1688 2007-05-09 02:22:30Z jerryhsu $
*/

void RGB2YCbCrDSPip(short *r_y, short *g_cb, short *b_cr, int row, int column) {
    int length = row * column;
    int i;
    int yC,cbC,crC;
    //int q_pt = 12;
    //int round = 1 << (q_pt - 1);
    for (i=0; i<length; i++) {
        yC = 1052 * r_y[i] + 2065 * g_cb[i] + 401 * b_cr[i];
        cbC = -607 * r_y[i] + -1192 * g_cb[i] + 1799 * b_cr[i];
        crC = 1799 * r_y[i] + -1507 * g_cb[i] + -292 * b_cr[i];
        /*yC = (yC + round) >> q_pt;
        cbC = (cbC + round) >> q_pt;
        crC = (crC + round) >> q_pt;*/
        yC = (yC + 2048) >> 12;
        cbC = (cbC + 2048) >> 12;
        crC = (crC + 2048) >> 12;
        r_y[i] = (short)yC + 16;
        g_cb[i] = (short)cbC + 128; 
        b_cr[i] = (short)crC + 128;
    }
}

