/* RGB to YCbCr in-place

Copyright (c) 1999-2008
Maryland DSPCAD Research Group, The University of Maryland at College Park 

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the above
copyright notice and the following two paragraphs appear in all copies
of this software.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

*/

//////////////////////////////////////////////////////////////////////////
//// RGB2YCbCr.c
/** RGB to YCbCr in-place
R,G,B 0~255 to Y,Cb,Cr 0~255 according to Matlab code
Y       0.2568   0.5041   0.0979           R     16
Cb = [  -0.1482 -0.2910   0.4392     ]     G +  128 
Cr      0.4392   -0.3678 -0.0714           B    128
  
rate
r_y: row*column (short)
g_cb: row*column (short)
b_cr: row*column (short)
  
@author Chia-Jui Hsu
@version $Id: RGB2YCbCrip.c 1688 2007-05-09 02:22:30Z jerryhsu $
*/

void RGB2YCbCrip(short *r_y, short *g_cb, short *b_cr, int row, int column) {
    int length = row * column;
    int i;
    short r,g,b;
    float y,cb,cr;
    for (i=0; i<length; i++) {
        r = r_y[i];
        g = g_cb[i];
        b = b_cr[i];
        y = 16 + 0.2568 * r + 0.5041 * g + 0.0979 * b;
        cb = 128 + -0.1482 * r + -0.2910 * g + 0.4392 * b;
        cr = 128 + 0.4392 * r + -0.3678 * g + -0.0714 * b;
        r_y[i] = (y-(short)y) >= 0.5 ? (short)y + 1: (short)y;
        g_cb[i] = (cb-(short)cb) >= 0.5 ? (short)cb + 1: (short)cb;
        b_cr[i] = (cr-(short)cr) >= 0.5 ? (short)cr + 1: (short)cr;
    }
}

