/* Twiddle factor generator for optimized FFT algorithm

Copyright (c) 1999-2008
Maryland DSPCAD Research Group, The University of Maryland at College Park 

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the above
copyright notice and the following two paragraphs appear in all copies
of this software.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

*/

#include <math.h>

//////////////////////////////////////////////////////////////////////////
//// TwiddleFactor
/** Twiddle factor generator for optimized FFT algorithm
For complex data, the real and imag part are interleaved in every two float data.
This code is based on TI SPRU657.pdf.

production rate
w: N*2 (float)
  
@author Chia-Jui Hsu
@version $Id: TwiddleFactor.c 1688 2007-05-09 02:22:30Z jerryhsu $
*/

void TwiddleFactor(float *w, int N) {
    int i, j, k;
    double x_t, y_t, theta1, theta2, theta3;
    const double PI = 3.141592654;
    for (j=1, k=0; j <= N>>2; j = j<<2) {
        for (i=0; i < N>>2; i+=j) {
            theta1 = 2*PI*i/N;
            x_t = cos(theta1);
            y_t = sin(theta1);
            w[k] = (float)x_t;
            w[k+1] = (float)y_t;
            theta2 = 4*PI*i/N;
            x_t = cos(theta2);
            y_t = sin(theta2);
            w[k+2] = (float)x_t;
            w[k+3] = (float)y_t;
            theta3 = 6*PI*i/N;
            x_t = cos(theta3);
            y_t = sin(theta3);
            w[k+4] = (float)x_t;
            w[k+5] = (float)y_t;
            k+=6;
        }
    }
}

