/* Wav File Reader

Copyright (c) 1999-2008
Maryland DSPCAD Research Group, The University of Maryland at College Park 

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the above
copyright notice and the following two paragraphs appear in all copies
of this software.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

*/

#include <stdio.h>

//////////////////////////////////////////////////////////////////////////
//// WavReader
/** Wave file reader.
It reads a 2-byte sample from each channel at a time and outputs them to 
output1 and output2. The output value is normalized in 1 to -1 range.

production rate:
output1 = 1
output2 = 1

@author Chia-Jui Hsu
@version $Id: WavReader.c 1688 2007-05-09 02:22:30Z jerryhsu $
*/

//FILE *readerFP;

void WavReader(float *output1, float *output2, FILE *readerFP) {

    short sample1, sample2;
    float factor = 32768;
    
    //if FILE has already been opened, use FILE pointer to read data directly.
    if (ftell(readerFP) > 0){
	fread(&sample1, sizeof(short), 1, readerFP);
	fread(&sample2, sizeof(short), 1, readerFP);
	//Send data out in two channels
	*output1 = (float)sample1 / factor;
	*output2 = (float)sample2 / factor;
    } else {
        // 1 byte
	char id[4];
        // 4 bytes
	unsigned long chunkSize, formatLength, sampleRate, avgBytesSec, dataSize;
        // 2 bytes
	short formatTag, channels, blockAlign, bitsPerSample;

	//readerFP = fopen(filename, "rb");
	//if (readerFP) {
	    fread(id, sizeof(char), 4, readerFP);
	    fread(&chunkSize, sizeof(unsigned long), 1, readerFP);	
	    fread(id, sizeof(char), 4, readerFP);
	    fread(id, sizeof(char), 4, readerFP);
	    fread(&formatLength, sizeof(unsigned long), 1, readerFP);
	    fread(&formatTag, sizeof(short), 1, readerFP);
	    //number of channles for this wave file
	    fread(&channels, sizeof(short),1,readerFP);
	    //the sampling rate for this wave file
	    fread(&sampleRate, sizeof(unsigned long), 1, readerFP);
	    //average bytes per second for accessing data in this wave file
	    fread(&avgBytesSec, sizeof(unsigned long), 1, readerFP);
	    //the block alignment for this wave file
	    fread(&blockAlign, sizeof(short), 1, readerFP);
	    //the number of sample per bits for data in this wave file
	    fread(&bitsPerSample, sizeof(short), 1, readerFP);
	    fread(id, sizeof(char), 4, readerFP);
	    //the data size 
	    fread(&dataSize, sizeof(unsigned long), 1, readerFP);
	    //calculate data time by using parameters obtained above
	    fread(&sample1, sizeof(short), 1, readerFP);
	    fread(&sample2, sizeof(short), 1, readerFP);
            
            //normalize            
	    //Send data out in two channels
	    *output1 = (float)sample1 / factor;
	    *output2 = (float)sample2 / factor;
	/*} else {
            printf("fopen error.\n");
            exit(0);
        }*/
    }
}

