/* Wave File Writer - single channel

Copyright (c) 1999-2008
Maryland DSPCAD Research Group, The University of Maryland at College Park 

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the above
copyright notice and the following two paragraphs appear in all copies
of this software.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

*/

#include <stdio.h>

//////////////////////////////////////////////////////////////////////////
//// WavWriter
/** Wave file writer - to single channel wave file
It write a 2-byte sample for single channel at a time.
The input value should be normalized in 1 to -1 range.
WavClose must be invoked in order to write formatSize and dataSize.

consumption rate
input = 1

@author Chia-Jui Hsu
@version $Id: WavWrite.c 1688 2007-05-09 02:22:30Z jerryhsu $
*/

void WavWrite(float *input, FILE* writerFP, unsigned int sampleRate) {

    short sample;
    float factor = 32768;

    //if FILE has already been opened, use FILE pointer to write data directly.
    if (ftell(writerFP) > 0){
        sample = (short)(*input * factor);
        fwrite(&sample, sizeof(short), 1, writerFP);        

    } else {
    //Specify required DAT-format values to its corresponding variables
        unsigned int formatSize = 0;
	unsigned int formatLength = 16;
	short formatTag = 1;
	unsigned int avgBytesSec = sampleRate * 2;
	short blockAlign = 2;
        short bitsPerSample = 16;
        short channels = 1;
        unsigned int dataSize = 0;
        
	fwrite("RIFF", sizeof(char), 4, writerFP);
	fwrite(&formatSize, sizeof(unsigned int), 1, writerFP);
	fwrite("WAVE", sizeof(char), 4, writerFP);
	fwrite("fmt ", sizeof(char), 4, writerFP);
	fwrite(&formatLength, sizeof(unsigned int), 1, writerFP);
	fwrite(&formatTag, sizeof(short), 1, writerFP);
	fwrite(&channels, sizeof(short), 1, writerFP);
	fwrite(&sampleRate, sizeof(unsigned int), 1, writerFP);
	fwrite(&avgBytesSec, sizeof(unsigned int), 1, writerFP);
	fwrite(&blockAlign, sizeof(short), 1, writerFP);
	fwrite(&bitsPerSample, sizeof(short), 1, writerFP);
	fwrite("data", sizeof(char), 4, writerFP);
	fwrite(&dataSize, sizeof(unsigned int), 1, writerFP);

        sample = (short)(*input * factor);
        fwrite(&sample, sizeof(short), 1, writerFP);        
    }
}


