/* Wave File Writer 

Copyright (c) 1999-2008
Maryland DSPCAD Research Group, The University of Maryland at College Park 

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the above
copyright notice and the following two paragraphs appear in all copies
of this software.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

*/

#include <stdio.h>

//////////////////////////////////////////////////////////////////////////
//// WavWriter
/** Wave file writer
It write a 2-byte sample for each channel at a time.
The input value should be normalized in 1 to -1 range.

@author Chia-Jui Hsu
@version $Id: WavWriter.c 1688 2007-05-09 02:22:30Z jerryhsu $
*/

//FILE *writerFP;
unsigned long dataSize;

void WavWriter(float *input1, float *input2, FILE *writerFP, 
        short channels, unsigned long sampleRate) {

    short sample1, sample2;
    float factor = 32768;
    unsigned long formatSize;

    //if FILE has already been opened, use FILE pointer to write data directly.
    if (ftell(writerFP) > 0){
        dataSize += 2*channels;
        formatSize = dataSize + 36;
        fseek(writerFP, 4, SEEK_SET);
        fwrite(&formatSize, sizeof(unsigned long), 1, writerFP);
        fseek(writerFP, 40, SEEK_SET);
        fwrite(&dataSize, sizeof(unsigned long), 1, writerFP);

        fseek(writerFP, 0, SEEK_END);
        //Read data in from two channel
        sample1 = (short)(*input1 * factor);
        sample2 = (short)(*input2 * factor);
        //Write data to the DAT-format wave file
        fwrite(&sample1, sizeof(short), 1, writerFP);        
        if (channels == 2) {
            fwrite(&sample2, sizeof(short), 1, writerFP);
        }
    } else {
    //Specify required DAT-format values to its corresponding variables
	unsigned long formatLength = 16;
	short formatTag = 1;
	//short channels = 2;
	//unsigned long sampleRate = 44100;
	unsigned long avgBytesSec = sampleRate * 2 * channels;
	short blockAlign = 2 * channels;
        short bitsPerSample = 16;
        
        dataSize = 2*channels;
	//formatSize = dataSize + 36;

	//writerFP = fopen(filename, "wb");
        //if (writerFP) {
	    fwrite("RIFF", sizeof(char), 4, writerFP);
	    fwrite(&formatSize, sizeof(unsigned long), 1, writerFP);
	    fwrite("WAVE", sizeof(char), 4, writerFP);
	    fwrite("fmt ", sizeof(char), 4, writerFP);
	    fwrite(&formatLength, sizeof(unsigned long), 1, writerFP);
	    fwrite(&formatTag, sizeof(short), 1, writerFP);
	    //number of channles for this wave file
	    fwrite(&channels, sizeof(short), 1, writerFP);
	    //the sampling rate for this wave file
	    fwrite(&sampleRate, sizeof(unsigned long), 1, writerFP);
	    //average bytes per second for accessing data in this wave file		
	    fwrite(&avgBytesSec, sizeof(unsigned long), 1, writerFP);
	    //the block alignment for this wave file
	    fwrite(&blockAlign, sizeof(short), 1, writerFP);
	    //the number of sample per bits for data in this wave file
	    fwrite(&bitsPerSample, sizeof(short), 1, writerFP);
	    fwrite("data", sizeof(char), 4, writerFP);
	    fwrite(&dataSize, sizeof(unsigned long), 1, writerFP);
	    //Read data in from two channel
	    sample1 = (short)(*input1 * factor);
	    sample2 = (short)(*input2 * factor);
	    //Write data to the DAT-format wave file
	    fwrite(&sample1, sizeof(short), 1, writerFP);
            if (channels == 2) {
                fwrite(&sample2, sizeof(short), 1, writerFP);
            }
	/*} else {
            printf("fopen error.\n");
            exit(0);
        }*/
    }
}


