/* Wave File Reader - stereo to single channel

Copyright (c) 1999-2008
Maryland DSPCAD Research Group, The University of Maryland at College Park 

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the above
copyright notice and the following two paragraphs appear in all copies
of this software.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

*/

#include <stdio.h>

//////////////////////////////////////////////////////////////////////////
//// WaveRead
/** Wave file reader - stereo to single channel.
It reads a sample from stereo wave file at a time and outputs only 
single channel to output. The output value is normalized in 1 to -1 range.
Each sample is restricted to be 2-byte.

production rate:
output =  length

@author Chia-Jui Hsu
@version $Id: WaveRead.c 1688 2007-05-09 02:22:30Z jerryhsu $
*/

short channels;

void WaveRead(float* output, FILE* readerFP, int length) {

    short sample;
    float factor = 32768;
    int i;
    
    //if FILE has already been opened, use FILE pointer to read data directly.
    if (ftell(readerFP) > 0) {
        for (i=0; i<length; i++) {
            fread(&sample, sizeof(short), 1, readerFP);
            if (channels == 2) {
                fseek(readerFP, sizeof(short), SEEK_CUR);
            }
    	    //normalize
	    output[i] = (float)sample / factor;
        }
    } else {
        fseek(readerFP, 22, SEEK_SET);
        fread(&channels, sizeof(short),1,readerFP);
        fseek(readerFP, 44, SEEK_SET);
        
        for (i=0; i<length; i++) {
            fread(&sample, sizeof(short), 1, readerFP);
            if (channels == 2) {
                fseek(readerFP, sizeof(short), SEEK_CUR);
            }
            //normalize
            output[i] = (float)sample / factor;
        }
    }
}

